#include "stdio.h"
#include "ctype.h"
#include "platform.h"
#include "mxchipWNet.h"
#include "mxchipWNet_HA.h"
#include "user_misc.h"
#include "flash_if.h"
#include "uart_protocol.h"

#define MAX_CLIENT 8
#define MAX_SITES 32
#define ONE_SECOND 1000
#define ONE_MINUTE 60*ONE_SECOND
#define MAX_UPGRADE_TIME 5*ONE_MINUTE// upgrade time should less than 5 minutes.

#define UART_CMD_MAX_LEN 1400
#define UART_CMD_LEN 12
#define IP_CMD_MAX_LEN 1460

#define UART_FRAM_START 0xAA
#define UART_FRAM_END 0x55
#define CONTROL_FLAG 0xBB
#define FRAM_FLAG 0x00BB

#define UAP_START_TIME_SHORT 5*ONE_SECOND
#define STATION_RETRY_SHORT 20*ONE_SECOND
#define SEND_TIMEOUT 200




enum {
  CMD_OK = 0,
  CMD_FAIL,
};

enum {
  INIT_STATE = 0, // first 20 seconds until wifi & tcp state changed
  DUAL_MODE_STATE, // After 20 seconds, wifi didn't connected
  NORMAL_WORK_STATE, // tcp connected.
  NORMAL_WORK_RETRY_STATE // TCP/STA disconnected, retry.
};

typedef  struct  _scan_tbl  
{  
    char ssid[32];  
    int rssi;  // sorted by rssi
}scan_tbl_t; 

#pragma pack(1)
typedef struct _upgrade_t {
  u8 md5[16];
  u32 len;
  u8 data[1];
}ota_upgrate_t;

#pragma pack(1)
typedef struct _udp_search {
  u8 ip[16];
  u8 mac[18];
  u8 ver[8]; 
}udp_search_st;


extern vu32 MS_TIMER;
extern vu32 SEC_TIMER;
extern vu32 TICK_TIMER;

static int wifi_disabled = 0;
static int need_reload = 0;
static int startScan = 0;
static int startStation = 0;
static int startAP = 0;
static int uap_start_tick = 0;
static u32 network_state = 0;
static u32 running_state = 0;

static int cloud_ip_addr = 0;
static u32 dns_pending = 0;
static u32 cloud_retry = 0;
static u32 cloud_heart_tick = 0;
static u32 new_configuration = 0;
static u32 cloud_enable = CLOUD_ENABLE;
static u32 uap_start_time = UAP_START_TIME;
static int ap_num = 0;

static u8 cloud_heart_content[] = {0xCC, 0x55};

extern midea_cmd_head_t mideaNetCmd, mideaUartCmd;

static u8 hugebuf[1600]; // cmd, fwd data are saved in this buffer
mxchipWNet_HA_st  *device_info;


//static void tcp_client_tick(void);
void delay_reload(void);
static void report_state(void);
static void tcp_recv_tick(void);
static void connect_to_cloud(void);
static void close_cloud(int retry_interval);
static int ip_cmd_process(unsigned char *buf, int cmd_len, int socket_fd);
static int search_cmd_process(unsigned char *buf, int cmd_len, const struct sockaddr_t *dest_addr, socklen_t addrlen);
static int ota_process(unsigned char *buf, int cmd_len, int *socket_fd);
static void fast_send(int sockfd, const void *buf, size_t len);
static void cloud_login(int fd);

int is_wifi_disabled(void)
{
  return wifi_disabled;
}

int is_network_state(int state)
{
  if ((network_state & state) == 0)
    return 0;
  else
    return 1;
}

static void report_state()
{
  u8 status;
  if(device_info->protocolVer ==1){
    uart_report_state();
  }
  else{
    get_status_uart(&status);
    status_changed_callback(status);
  }
}

void set_network_state(int state, int on)
{
  int i;
  if (on)
    network_state |= state;
  else {
    network_state &= ~state;
    if (state == STA_CONNECT)
      network_state &= ~REMOTE_CONNECT;
  }
	
  if ((state == STA_CONNECT) || (state == REMOTE_CONNECT))  
    report_state();

  if(device_info->mode == Type_FORCE ){
    if(new_configuration && (state == STA_CONNECT) && on){
      for(i=0;i<MAX_CLIENT;i++) {
        if (device_info->tcpClient_fd[i] != 0)
          fast_send(device_info->tcpClient_fd[i], "Config Success!", 16);
        }
      new_configuration = 0;
      updateConfiguration(device_info);
      msleep(500);
			if(is_network_state(UAP_START)){ //Exit uap mode if configutation is success
				uap_close();
			}
    }
  }
  else if(device_info->mode == Type_MANUAL){
    if(new_configuration && (state == STA_CONNECT) && on){
      for(i=0;i<MAX_CLIENT;i++) {
        if (device_info->tcpClient_fd[i] != 0)
          fast_send(device_info->tcpClient_fd[i], "Config Success!", 16);
        }
      new_configuration = 0;
      device_info->conf.interface = Station; //Enter the station mode in  next power on if config is right 
      updateConfiguration(device_info);
      msleep(500);
    }
    
    if(is_network_state(UAP_START)) //Back to previous status in manual mode
      sta_close();
  }
  else{ //auto mode
    switch(running_state) {
    case INIT_STATE:
    case NORMAL_WORK_RETRY_STATE:
      if (on) {
        if (state == REMOTE_CONNECT)  //connected to remote server
          running_state = NORMAL_WORK_STATE;

        if (state == UAP_START) 
          running_state = DUAL_MODE_STATE;
      }
      else{
        if (state == STA_CONNECT) {
          uap_start_time = MS_TIMER + UAP_START_TIME_SHORT;
        }
      }
      break;
    case DUAL_MODE_STATE:
      if ((state == STA_CONNECT) && on) {
        //Send to the clients that the new configuration is right
        if(new_configuration){
          for(i=0;i<MAX_CLIENT;i++) {
            if (device_info->tcpClient_fd[i] != 0)
              fast_send(device_info->tcpClient_fd[i], "Config Success!", 16);
          }
          updateConfiguration(device_info);
          new_configuration = 0;
        }		
        msleep(500);
        delay_reload();
        return; // do not report tcp connect, cause we will reboot system.
      }
      break;

    case NORMAL_WORK_STATE:
      if (!on && ((state == STA_CONNECT) || (state == REMOTE_CONNECT))) {
        if (state == STA_CONNECT) {
          uap_start_time = MS_TIMER + UAP_START_TIME_SHORT;
        }
        running_state = NORMAL_WORK_RETRY_STATE;
      }
      break;
      
    default:
      break;
    }

  }
}

void delay_reload()
{
  need_reload = 1;
}


void formatMACAddr(void *destAddr, void *srcAddr)
{
  sprintf((char *)destAddr, "%c%c:%c%c:%c%c:%c%c:%c%c:%c%c",\
					toupper(*(char *)srcAddr),toupper(*((char *)(srcAddr)+1)),\
					toupper(*((char *)(srcAddr)+2)),toupper(*((char *)(srcAddr)+3)),\
					toupper(*((char *)(srcAddr)+4)),toupper(*((char *)(srcAddr)+5)),\
					toupper(*((char *)(srcAddr)+6)),toupper(*((char *)(srcAddr)+7)),\
					toupper(*((char *)(srcAddr)+8)),toupper(*((char *)(srcAddr)+9)),\
					toupper(*((char *)(srcAddr)+10)),toupper(*((char *)(srcAddr)+11)));
}

void socket_connected(int fd)
{
  if(fd==device_info->cloud_fd){
#ifdef CLOUD_SSL
    if (setSSLmode(1, device_info->cloud_fd)!= MXCHIP_SUCCESS){
      close_cloud(CLOUD_RETRY);
      return;
    }

#endif
    set_network_state(REMOTE_CONNECT, 1);
    cloud_login(fd);
  }
}

void RptConfigmodeRslt(network_InitTypeDef_st *nwkpara)
{
  if(nwkpara == NULL){
    system_reload();
    }
  else{
    memcpy(device_info->conf.sta_ssid, nwkpara->wifi_ssid, sizeof(device_info->conf.sta_ssid));
    memcpy(device_info->conf.sta_key, nwkpara->wifi_key, sizeof(device_info->conf.sta_key));
    updateConfiguration(device_info);
    system_reload();
  }
}

void userWatchDog(void)
{
}

void WifiStatusHandler(int event)
{
  switch (event) {
    case MXCHIP_WIFI_UP:
			set_network_state(STA_CONNECT,1);
      break;
    case MXCHIP_WIFI_DOWN:
			set_network_state(STA_CONNECT,0);
      break;
    case MXCHIP_UAP_UP:
      set_network_state(UAP_START,1);
      break;
    case MXCHIP_UAP_DOWN:
      set_network_state(UAP_START,0);
      break;  
    default:
      break;
  }
  return;
}

void ApListCallback(UwtPara_str *pApList)
{
  ap_num = pApList->ApNum;
  memcpy(&hugebuf[sizeof(mxchip_cmd_head_t)-1], pApList->ApList, ap_num*sizeof(ApList_str));
  startScan = 0;
}

void NetCallback(net_para_st *pnet)
{
  strcpy((char *)device_info->status.ip, pnet->ip);
  strcpy((char *)device_info->status.mask, pnet->mask);
  strcpy((char *)device_info->status.gw, pnet->gate);
  strcpy((char *)device_info->status.dns, pnet->dns);
}

///////////////////////////////////////////////////////////////////

/* Sysytem initilize */
void mxchipWNet_HA_init(void) 
{
  net_para_st para;
  midea_type type;
	device_info = (mxchipWNet_HA_st *)malloc(sizeof(mxchipWNet_HA_st));
  memset(device_info, 0, sizeof(mxchipWNet_HA_st)); 

  SystemCoreClockUpdate();
	mxchipInit();
  hal_uart_init();
  getNetPara(&para, Station);
  formatMACAddr((void *)device_info->status.mac, &para.mac);
  strcpy((char *)device_info->status.ip, (char *)&para.ip);
  strcpy((char *)device_info->status.mask, (char *)&para.mask);
  strcpy((char *)device_info->status.gw, (char *)&para.gate);
  strcpy((char *)device_info->status.dns, (char *)&para.dns);

  readConfiguration(device_info);
  
  while(MS_TIMER<3000); //wait the midea board boot
  type = get_midea_type();
  if(type == Type_unknow)          
    type = device_info->conf.mideaType;
  else if(device_info->conf.mideaType != type){
    device_info->conf.mideaType = type;
    updateConfiguration(device_info);
  }

  if(type == Type_Oversea_FORCE ){
    device_info->protocolVer = 2;
    device_info->mode = Type_FORCE;
  }
  else if(type == Type_Oversea_MANUAL){
    device_info->protocolVer = 2;
    device_info->mode = Type_MANUAL; 
  }
  else if(type == Type_RD_AUTO){
    device_info->protocolVer = 2;
    device_info->mode = Type_AUTO; 
  }
  else{
    device_info->protocolVer = 1;
    device_info->mode = Type_AUTO; 
  }
  
  device_info->tcpServer_fd = 0;
  device_info->tcpClient_fd = (int *)malloc(4 * MAX_CLIENT);
  device_info->tcpServer_fd = 0;
  device_info->udpSearch_fd = 0;
  memset(device_info->tcpClient_fd, 0x0, sizeof(4 * MAX_CLIENT));

  if(device_info->mode != Type_MANUAL){
		sta_start();
	}
	else {
		if(device_info->conf.interface == Soft_AP)
			uap_start();
		else
			sta_start();
	}

  setSslMaxlen(2*1024);
}

void sta_start(void)
{
  network_InitTypeDef_st wNetConfig;
  
  if(startStation)
    return;

  wNetConfig.wifi_mode = Station;
  strcpy(wNetConfig.wifi_ssid, device_info->conf.sta_ssid);
  strcpy(wNetConfig.wifi_key, device_info->conf.sta_key);
  wNetConfig.dhcpMode = DHCP_Client;
  strcpy(wNetConfig.local_ip_addr, (char*)device_info->conf.ip);
  strcpy(wNetConfig.net_mask, (char*)device_info->conf.mask);
  strcpy(wNetConfig.gateway_ip_addr, (char*)device_info->conf.gw);
  strcpy(wNetConfig.dnsServer_ip_addr, (char*)device_info->conf.dns);
  wNetConfig.wifi_retry_interval = 100;
  ps_enable();
  StartNetwork(&wNetConfig);
  startStation = 1;
	cloud_enable = 1;
}

void sta_close(void)
{
  if(startStation == 0)
    return;
  /* close sta */
  close_cloud(-1);
  sta_disconnect();
  startStation = 0;
}

void uap_start(void)
{
  network_InitTypeDef_st wNetConfig;
  
  if(startAP)
    return;

  wNetConfig.wifi_mode = Soft_AP;
  sprintf(wNetConfig.wifi_ssid, "%s_%c%c%c%c", device_info->conf.uap_ssid,\
                  device_info->status.mac[12], device_info->status.mac[13],
                  device_info->status.mac[15], device_info->status.mac[16]);
  strcpy(wNetConfig.wifi_key, device_info->conf.uap_key);
  wNetConfig.dhcpMode = DHCP_Server;
  strcpy(wNetConfig.local_ip_addr, UAP_IP_ADDRESS);
  strcpy(wNetConfig.net_mask, UAP_MASK_ADDRESS);
  strcpy(wNetConfig.gateway_ip_addr, UAP_GW_ADDRESS);
  strcpy(wNetConfig.dnsServer_ip_addr, UAP_DNS_ADDRESS);
  strcpy(wNetConfig.address_pool_start, UAP_DHCP_SERVER_START);
  strcpy(wNetConfig.address_pool_end, UAP_DHCP_SERVER_END);
  StartNetwork(&wNetConfig);
  startAP = 1;

  /*We need a time when AP is started, for uap mode lasts only 3 minutes in forced station mode*/
  uap_start_tick = MS_TIMER; 
}

void uap_close(void)
{
  if(startAP == 0)
    return;
  uap_stop();
  startAP = 0;  
}


/* main tick */
void mxchipWNet_HA_tick(void)
{
  mxchipTick();
  if (!is_wifi_disabled()) {
    if (is_network_state(STA_CONNECT) == 0 && is_network_state(UAP_START) == 0 && device_info->mode == Type_AUTO) {
      if (MS_TIMER > uap_start_time) {
        if(device_info->cloud_fd > 0) {  //Donot retry to cloud, as the station is disconnected, reboot if station is recovered
          close(device_info->cloud_fd);
          cloud_enable = 0;
        }
        uap_start();
      }
    }

    if(is_network_state(STA_CONNECT) && cloud_enable && MS_TIMER > cloud_retry)
      connect_to_cloud();

    /*Send heartbeat to cloud in protocol v2*/
    if(is_network_state(REMOTE_CONNECT) && MS_TIMER>cloud_heart_tick && device_info->protocolVer != 1){ 
      fast_send(device_info->cloud_fd, cloud_heart_content, sizeof(cloud_heart_content));
      cloud_heart_tick = MS_TIMER + CLOUD_HEART_INTERVAL;
    }

    /* In force station mode, exit uap after upa has setup for 3 minitus*/
    if((MS_TIMER - uap_start_tick) > 3*ONE_MINUTE && device_info->mode == Type_FORCE && is_network_state(UAP_START)){  
      uap_close();
    }

    tcp_recv_tick();
  }

  if (need_reload == 1) {
    msleep(500);
    NVIC_SystemReset();
  }
}

void dns_ip_set(u8 *hostname, u32 ip)
{
	if((int)ip == -1){
		cloud_ip_addr = 0;
		cloud_retry = MS_TIMER + CLOUD_RETRY;
	}
	else
		cloud_ip_addr = ip;
	
  dns_pending = 0;
}

static void connect_to_cloud(void)
{
  int opt = 0;
  struct sockaddr_t addr;

  if(cloud_ip_addr == 0 && dns_pending == 0){  //DNS function
    cloud_ip_addr = dns_request((char *)device_info->conf.server_domain);
    if(cloud_ip_addr == -1){
      cloud_ip_addr = 0; //dns failed, retry again
      goto cloud_error;  
    }
    else if (cloud_ip_addr == 0){  //DNS pending, waiting for callback
      dns_pending = 1;
      return;
    }
	}

  if( device_info->cloud_fd == 0 && (u32)cloud_ip_addr>0){
    device_info->cloud_fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    setsockopt(device_info->cloud_fd,0,SO_BLOCKMODE,&opt,4);
    addr.s_ip = cloud_ip_addr; 
    addr.s_port = device_info->conf.server_port;
    if (connect(device_info->cloud_fd, &addr, sizeof(addr))!=0) 
      goto cloud_error; 
  }
	return;
cloud_error:
  close_cloud(CLOUD_RETRY);
  return;
}

static void close_cloud(int retry_interval)
{
  close(device_info->cloud_fd);
  device_info->cloud_fd = 0;
  cloud_ip_addr = 0;

	//cloud may not be connected yet in unblock mode, 
	if(is_network_state(REMOTE_CONNECT) == 1) 
		set_network_state(REMOTE_CONNECT, 0);
  cloud_retry = MS_TIMER + retry_interval;
  if(retry_interval == -1)
    cloud_enable = 0;
}


static void tcp_recv_tick(void)
{
  int i, j, len;
  fd_set readfds;
  struct timeval_t t;
  struct sockaddr_t addr;
  socklen_t addrLen;
  int optval = 1000;
  
  
  FD_ZERO(&readfds);
  t.tv_sec = 0;
  t.tv_usec = 1000;
  
  if (device_info->tcpServer_fd==0) {
    device_info->tcpServer_fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

    addr.s_port = TCP_SERVER_PORT;
    bind(device_info->tcpServer_fd, &addr, sizeof(addr));
    listen(device_info->tcpServer_fd, 0);
    
    memset(device_info->tcpClient_fd, 0, 4 * MAX_CLIENT);
  } else {
    FD_SET(device_info->tcpServer_fd, &readfds);
    for(i=0;i<MAX_CLIENT;i++) {
      if (device_info->tcpClient_fd[i] != 0)
        FD_SET(device_info->tcpClient_fd[i], &readfds);
    }
  }

  if(device_info->udpSearch_fd==0){
    device_info->udpSearch_fd = socket(AF_INET, SOCK_DGRM, IPPROTO_UDP);
    addr.s_port = UDP_BROADCAST_PORT;
    bind(device_info->udpSearch_fd, &addr, sizeof(addr));
  }
  else{
    FD_SET(device_info->udpSearch_fd, &readfds);
  }

  if(device_info->cloud_fd > 0)
    FD_SET(device_info->cloud_fd, &readfds);

  select(1, &readfds, NULL, NULL, &t);

  for(i=0;i<MAX_CLIENT;i++) {
    if (device_info->tcpClient_fd[i] != 0) {
      if (FD_ISSET(device_info->tcpClient_fd[i], &readfds)) {
        len = recv(device_info->tcpClient_fd[i], hugebuf, IP_CMD_MAX_LEN, 0);
        if(len==-1){
          close(device_info->tcpClient_fd[i]);
          device_info->tcpClient_fd[i] = 0;
        }
        else
          ip_cmd_process(hugebuf, len, device_info->tcpClient_fd[i]);
      }
    }
  }
  
  if (FD_ISSET(device_info->tcpServer_fd, &readfds)) {
    j = accept(device_info->tcpServer_fd, &addr, &len);
    if (j > 0) {
      for(i=0;i<MAX_CLIENT;i++) {
        if ( device_info->tcpClient_fd[i] == 0) {
           device_info->tcpClient_fd[i] = j;
          setsockopt(j, 0, SO_SNDTIMEO, &optval, 4);
          break;
        }
      }
    }
  }

  if(device_info->cloud_fd > 0){
    if (FD_ISSET(device_info->cloud_fd, &readfds)) {
      len = recv(device_info->cloud_fd, hugebuf, IP_CMD_MAX_LEN, 0);
      if(len==-1)
        close_cloud(0);
      else
        ip_cmd_process(hugebuf, len, device_info->cloud_fd);
    }
  }

   if (FD_ISSET(device_info->udpSearch_fd, &readfds)) {
     len = recvfrom(device_info->udpSearch_fd, hugebuf, IP_CMD_MAX_LEN, 0, &addr, &addrLen);
     search_cmd_process(hugebuf, len, &addr, sizeof(struct sockaddr_t));
  } 
}


static int search_cmd_process(unsigned char *buf, int cmd_len, const struct sockaddr_t *dest_addr, socklen_t addrlen)
{
  u32 len=0;
  mxchip_cmd_head_t *p_reply;
  udp_search_st *data;
  u16 cksum, cmd;
    
  if(device_info->udpSearch_fd == 0)
    return 0;

  if ((buf[0] != CONTROL_FLAG) || (buf[1] != 0))
    return 0;

  if (check_sum(hugebuf, cmd_len) != 1)
    return 0;
    
  p_reply = (mxchip_cmd_head_t *)buf; //Use the same data block to save the RAM
  data = (udp_search_st *)(p_reply->data);
  p_reply->cmd_status = CMD_OK;
  cmd = p_reply->cmd;
  p_reply->cmd |= 0x8000;
  if(cmd == CMD_SEARCH){
    p_reply->datalen = sizeof(udp_search_st);
    memcpy(data->ip, device_info->status.ip, 16);
    memcpy(data->mac, device_info->status.mac, 18);
    sprintf((char *)data->mac, "%08x", FW_VERSION);
    len = sizeof(mxchip_cmd_head_t) + 1 + p_reply->datalen;
    cksum = calc_sum(hugebuf, len-2);
    hugebuf[len-2] = cksum & 0x0ff;
    hugebuf[len-1] = (cksum & 0xff00) >> 8;
    sendto(device_info->udpSearch_fd, (u8*)p_reply, 0, \
          sizeof(mxchip_cmd_head_t) + 1 + p_reply->datalen, dest_addr, addrlen);
  }
  return 0;
}

static void mac_str2int(char *str, u8 *mac)
{
  int i, j;
  char mac_tmp[12];

  for(i=0, j=0; i<12; i++,j++){
    mac_tmp[i] = str[j];
    if(str[j+1]==':')
      j++;
  }
  str2hex((u8 *)mac_tmp, (u8 *)mac, 6);
}

static int ip_cmd_process(unsigned char *buf, int cmd_len, int socket_fd)
{
  u32 len=0;
  mxchip_cmd_head_t *p_reply;
  midea_cmd_head_t  *p_reply2;
  u16 cksum, cmd;
  u8 midea_mac[6];
    
  p_reply2 = (midea_cmd_head_t *)buf;

  //Midea protocol: Net->UART
	if(device_info->protocolVer == 1){
    if (buf[0] == 0xAA) { // UART forward
      mac_str2int((char*)(device_info->status.mac), midea_mac);
      if (memcmp(midea_mac, &buf[2], 6) == 0) {
        wifi_input(buf, 31);
        msleep(200);
        if(cmd_len - 31 > 0)
          ip_cmd_process(&buf[31], cmd_len-31, socket_fd);
      }
    }
  }
  else{
    if(memcmp(p_reply2, &mideaNetCmd, 6)==0 && p_reply2->lead_code == 0xAA){ //Midea Net->UART
      len = sizeof(midea_cmd_head_t)+p_reply2->datalen;
      wifi_input(buf, len);
      if(cmd_len-len > 0)
        ip_cmd_process(&buf[len], cmd_len-len, socket_fd);
    }
  }

	//MXCHIP protocol: Net commands
  if ((buf[0] != CONTROL_FLAG) || (buf[1] != 0))
    return 0;

  if (check_sum(hugebuf, cmd_len) != 1)
    return 0;
    
  p_reply = (mxchip_cmd_head_t *)buf;
  p_reply->cmd_status = CMD_OK;
  cmd = p_reply->cmd;
  p_reply->cmd |= 0x8000;
  switch (cmd) {
    case CMD_READ_VERSION:
      p_reply->datalen = 8;
      sprintf((char *)p_reply->data, "%08x", FW_VERSION);
      break;
      
    case CMD_READ_CONFIG:
      p_reply->datalen = CONFIG_DATA_SIZE + 18;// 18 is mac address
      memcpy(p_reply->data, device_info->conf.uap_ssid, CONFIG_DATA_SIZE);
      memcpy((char *)&p_reply->data[CONFIG_DATA_SIZE], device_info->status.mac,18);
      break;

    case CMD_WRITE_CONFIG:
      if (p_reply->datalen < CONFIG_DATA_SIZE)
        p_reply->cmd_status = CMD_FAIL;
      else {
        if(device_info->mode == Type_MANUAL && MS_TIMER>3*ONE_MINUTE){
          p_reply->cmd_status = CMD_FAIL;
          break; //In manual mode, should not set new config after 3 minutes
        }
        memcpy(device_info->conf.uap_ssid, p_reply->data, CONFIG_DATA_SIZE);
				//strcpy((char*)device_info->conf.server_domain, DEFAULT_REMOTE_ADDR); //debug 
        new_configuration = 1;

        sta_close();
        sta_start();
        /*Use the new settings to connect AP*/
        //delay_reload();
      }
      p_reply->datalen = 0;
      break;

    case CMD_SCAN:
      mxchipStartScan();
      startScan = 1;

      while(startScan == 1){
        mxchipTick();
      }
      p_reply->datalen = ap_num*sizeof(ApList_str);
      break;

    case CMD_OTA:
      ota_process(buf, cmd_len, &socket_fd);
      p_reply->datalen = 0;
      return 0; //send ack in ota_process

    case CMD_NET2COM:
      wifi_input(buf, cmd_len);
			break;

    default:
      return 0;

  }
  
  len = sizeof(mxchip_cmd_head_t) + 1 + p_reply->datalen;
  cksum = calc_sum(hugebuf, len-2);
  hugebuf[len-2] = cksum & 0x0ff;
  hugebuf[len-1] = (cksum & 0xff00) >> 8;
  fast_send(socket_fd, (u8*)p_reply, len);
	if(cmd_len-len > 0) //Still has data needs to process
		ip_cmd_process(&buf[len], cmd_len-len, socket_fd);

  return 0;
}

static int ota_process(unsigned char *buf, int cmd_len, int *socket_fd)
{
  mxchip_cmd_head_t *p_control_cmd;
  ota_upgrate_t *p_upgrade;
  u8 * p_bin;
  int bin_len, total_len, head_len;
  u32 end_time = MS_TIMER + MAX_UPGRADE_TIME;
  u32 flash_addr = UPDATE_START_ADDRESS;
  u8 md5_ret[16];
  mxchip_cmd_head_t cmd_ack;
  fd_set readfds;
  struct timeval_t t;

  memset(&cmd_ack, 0, sizeof(cmd_ack));
  cmd_ack.cmd_status = CMD_FAIL;
  p_control_cmd = (mxchip_cmd_head_t *)buf;
  cmd_ack.flag = p_control_cmd->flag;
  cmd_ack.cmd = p_control_cmd->cmd | 0x8000;
  head_len = sizeof(mxchip_cmd_head_t) + sizeof(ota_upgrate_t) - 2;
  if (cmd_len < head_len){
    goto CMD_REPLY;
  }
  FLASH_If_Init();
  p_upgrade = (ota_upgrate_t*)(p_control_cmd->data);

  p_bin = p_upgrade->data;
  total_len = p_upgrade->len;
  bin_len = cmd_len - head_len;
  total_len -= bin_len;

  if (bin_len>0)
    FLASH_If_Write(&flash_addr, (uint32_t *)p_bin, bin_len);
  delay_reload(); //Reset module if OTA is finished

  while (total_len>0) {
    mxchipTick();
  
    FD_ZERO(&readfds);
    t.tv_sec = 0;
    t.tv_usec = 1000;
    FD_SET(*socket_fd, &readfds);
    select(1, &readfds, NULL, NULL, &t);

    if (FD_ISSET(*socket_fd, &readfds)) {
      bin_len = recv(*socket_fd, (char*)p_bin, 1024, 0);
      if(bin_len == -1){       
        if(*socket_fd == device_info->cloud_fd)
          close_cloud(0);
        else{
          close(*socket_fd);
          *socket_fd = 0;
        }
        
        return 0; //Net error, donot need to send ack
      }
      else{
        FLASH_If_Write(&flash_addr, (uint32_t *)p_bin, bin_len);
				total_len-=bin_len;
			}
    }
  }

  md5_hex((u8 *)UPDATE_START_ADDRESS, flash_addr - UPDATE_START_ADDRESS, md5_ret);
  if(memcmp(md5_ret, p_upgrade->md5, 16) != 0) {
    FLASH_Lock();
    goto CMD_REPLY;
  }

  memset(&device_info->conf.bootTable, 0, sizeof(boot_table_t));
  device_info->conf.bootTable.length = flash_addr - UPDATE_START_ADDRESS;
  device_info->conf.bootTable.start_address = UPDATE_START_ADDRESS;
  device_info->conf.bootTable.type = 'A';
  device_info->conf.bootTable.upgrade_type = 'U';
  updateConfiguration(device_info);
  cmd_ack.cmd_status = CMD_OK;
  
CMD_REPLY:
  fast_send(*socket_fd, (u8 *)&cmd_ack, sizeof(cmd_ack) + 1 + cmd_ack.datalen);
  return 0;
}

/* Get system status */
void get_status_uart(u8 *status)
{
  u8 tmp;
  tmp = 0;

  if(startStation == 1){
    if(is_network_state(STA_CONNECT))
      tmp = (tmp&~(0x3<<0))|(2<<0);    //Station connected
    else
      tmp = (tmp&~(0x3<<0))|(3<<0);    //Station connecting
  }
  else{
    if(is_network_state(STA_CONNECT))
      tmp = (tmp&~(0x3<<0))|(1<<0);    //Station disconnecting
    else
      tmp = (tmp&~(0x3<<0))|(0<<0);    //Station disconnected
  }

  if(startAP == 1){
    if(is_network_state(UAP_START))
      tmp = (tmp&~(0x3<<2))|(2<<2);    //AP connected
    else
      tmp = (tmp&~(0x3<<2))|(3<<2);    //AP connecting
  }
  else{
    if(is_network_state(UAP_START))
      tmp = (tmp&~(0x3<<2))|(1<<2);    //AP disconnecting
    else
      tmp = (tmp&~(0x3<<2))|(0<<2);    //AP disconnected
  }
  *status = tmp;
}

/* Get system status */
void get_status(mxchip_state_t *cmd)
{
  u16 cksum;
  sta_ap_state_t ap_state;

  cmd->flag = 0x00BB;
  cmd->cmd = 0x8008;
  cmd->cmd_status = 0;
  cmd->datalen = sizeof(mxchip_state_t) - 10;

  device_info->status.uap_state = is_network_state(UAP_START);
  device_info->status.sta_state = is_network_state(STA_CONNECT);
  if (is_network_state(STA_CONNECT) == 1)
    device_info->status.tcp_client = is_network_state(REMOTE_CONNECT) + 1;
  else
    device_info->status.tcp_client = 0;

  CheckNetLink(&ap_state);
  device_info->status.signal = ap_state.wifi_strength;
  memcpy(&cmd->status, &device_info->status, sizeof(current_state_t));

  cksum = calc_sum(cmd, sizeof(mxchip_state_t) - 2);
  cmd->cksum = cksum;
}

/* Back to default configuration and reload */
void system_default(void)
{
}

/* Wifi Power down*/
void wifi_disable(void)
{
	int i;
  for(i=0;i<MAX_CLIENT;i++) {
    if (device_info->tcpClient_fd[i] != 0)
			close(device_info->tcpClient_fd[i]);
  }

  if(device_info->cloud_fd != 0){
		close(device_info->cloud_fd);
		cloud_enable = 0;
	}
	wlan_disconnect();
	wifi_power_down();
}
/* Output data by wifi */
void wifi_output(u8 *buf, int len)
{
  int i;
  for(i=0;i<MAX_CLIENT;i++) {
    if (device_info->tcpClient_fd[i] != 0)
      fast_send(device_info->tcpClient_fd[i], buf, len);
  }

  if(device_info->cloud_fd != 0 && is_network_state(REMOTE_CONNECT))
    fast_send(device_info->cloud_fd, buf, len);
}

void wifi_input(u8 *buf, int len)
{
    hal_uart_send_data(buf, len);
		hal_uart_tx_tick();
}


void fast_send(int sockfd, const void *buf, size_t len)
{
  int start_tick = MS_TIMER;
  if(sockfd == device_info->cloud_fd && !is_network_state(REMOTE_CONNECT))
    return;
  while(tx_buf_size(sockfd)<(int)len){
    if(start_tick + SEND_TIMEOUT < (int)MS_TIMER)
      return;
    msleep(20);
  }
  send(sockfd, buf, len, 0);
  if(sockfd == device_info->cloud_fd)
    cloud_heart_tick = MS_TIMER + CLOUD_HEART_INTERVAL;
}

//前缀(0xA0）+ 版本号(0x01) + MAC地址字符串(12 bytes) + MD5(MAC地址字符串+key) (16 bytes 哈唏值) + 路由器SSID(32 bytes) + 路由器BSSID (12 bytes)
// 其中key是由我们随机生成的长字符串，可以定义为：
// f2919339a225efasbaef3b87e2cbfe641332a152


void cloud_login(int fd)
{
  cloud_login_t loginData;
  int i, j;
  char key[]="f2919339a225efasbaef3b87e2cbfe641332a152";
  char mac[13];
  char bssid[13];
  char mac_key[53];
  sta_ap_state_t ap_state;

  if(device_info->protocolVer == 1)
    return;

  for(i=0, j=0; i<12; i++, j++){
    if(device_info->status.mac[j]==':')
      j++;
    mac[i]=device_info->status.mac[j];
  }
	mac[12]=0;

  loginData.lead_code = 0xA0;
  loginData.version = FIRMWARE_VERSION;
  memcpy(loginData.mac, mac, 12);
  sprintf(mac_key, "%s%s", mac, key);
  md5_hex((u8 *)mac_key, 52, loginData.security);

  CheckNetLink(&ap_state);
  memcpy(loginData.ssid, ap_state.ssid, 32);
  for(i=0, j=0; i<6; i++, j=j+2)
    sprintf(&bssid[j], "%2x", ap_state.bssid[i]);
		
  memcpy(loginData.bssid, bssid, 12);

  fast_send(fd, &loginData, sizeof(cloud_login_t));
}













